// Copyright (C) 2009-2015 Specific Purpose Software GmbH
// GNU Lesser General Public License, version 2.1

#ifndef CALCULATOR_H
#define CALCULATOR_H

#include "listswitchable.h"

/*namespace Calculator {
}*/


/*template<typename T> class Calculator : public ListSwitchable<T>
{
public:
    using ListSwitchable<T>::ListSwitchable;


};*/

#endif // CALCULATOR_H
