// Copyright (C) 2009-2015 Specific Purpose Software GmbH
// GNU Lesser General Public License, version 2.1

#ifndef SPIRALSCALCULATOR_H
#define SPIRALSCALCULATOR_H

#include "listswitchable.h"

/*template<typename T> class SpiralsCalculator : public ListSwitchable<T>
{
public:
    using ListSwitchable<T>::ListSwitchable;

};*/

#endif // SPIRALSCALCULATOR_H
